@@ -55,25 +55,8 @@ typedef struct
         GObjectClass   parent_class;
 
         /* methods */
-        gboolean (*create_authority)          (GdmDisplay *display);
-        gboolean (*add_user_authorization)    (GdmDisplay *display,
-                                               const char *username,
-                                               char      **filename,
-                                               GError    **error);
-        gboolean (*remove_user_authorization) (GdmDisplay *display,
-                                               const char *username,
-                                               GError    **error);
-        gboolean (*set_slave_bus_name)        (GdmDisplay *display,
-                                               const char *name,
-                                               GError    **error);
-        gboolean (*prepare)                   (GdmDisplay *display);
-        gboolean (*manage)                    (GdmDisplay *display);
-        gboolean (*finish)                    (GdmDisplay *display);
-        gboolean (*unmanage)                  (GdmDisplay *display);
-        void     (*get_timed_login_details)   (GdmDisplay *display,
-                                               gboolean   *enabled,
-                                               char      **username,
-                                               int        *delay);
+        gboolean (*prepare) (GdmDisplay *display);
+        void     (*manage)  (GdmDisplay *self);
 } GdmDisplayClass;
 
 typedef enum
@@ -130,19 +113,18 @@ gboolean            gdm_display_get_seat_id                    (GdmDisplay *disp
 gboolean            gdm_display_is_local                       (GdmDisplay *display,
                                                                 gboolean   *local,
                                                                 GError    **error);
-gboolean            gdm_display_get_timed_login_details        (GdmDisplay *display,
+void                gdm_display_get_timed_login_details        (GdmDisplay *display,
                                                                 gboolean   *enabled,
                                                                 char      **username,
-                                                                int        *delay,
-                                                                GError    **error);
+                                                                int        *delay);
 gboolean            gdm_display_is_initial                     (GdmDisplay  *display,
                                                                 gboolean    *initial,
                                                                 GError     **error);
 
-/* exported but protected */
-gboolean            gdm_display_get_x11_cookie                 (GdmDisplay *display,
-                                                                GArray     **x11_cookie,
-                                                                GError    **error);
+gboolean            gdm_display_get_x11_cookie                 (GdmDisplay  *display,
+                                                                const char **x11_cookie,
+                                                                gsize       *x11_cookie_size,
+                                                                GError     **error);
 gboolean            gdm_display_get_x11_authority_file         (GdmDisplay *display,
                                                                 char      **filename,
                                                                 GError    **error);
@@ -153,9 +135,13 @@ gboolean            gdm_display_add_user_authorization         (GdmDisplay *disp
 gboolean            gdm_display_remove_user_authorization      (GdmDisplay *display,
                                                                 const char *username,
                                                                 GError    **error);
-gboolean            gdm_display_set_slave_bus_name             (GdmDisplay *display,
-                                                                const char *name,
-                                                                GError    **error);
+
+void                gdm_display_set_up_greeter_session         (GdmDisplay  *display,
+                                                                char       **username);
+void                gdm_display_start_greeter_session          (GdmDisplay  *display);
+void                gdm_display_stop_greeter_session           (GdmDisplay  *display);
+
+gboolean            gdm_display_connect                        (GdmDisplay *self);
 
 G_END_DECLS
 
