@@ -94,7 +94,6 @@ gdm_local_display_prepare (GdmDisplay *display)
         char          *session_class;
         char          *session_type;
         gboolean       doing_initial_setup = FALSE;
-        gboolean       failed = FALSE;
 
         seat_id = NULL;
 
@@ -109,8 +108,6 @@ gdm_local_display_prepare (GdmDisplay *display)
                 goto out;
         }
 
-        g_debug ("doing initial setup? %s", doing_initial_setup? "yes" : "no");
-
         if (!doing_initial_setup) {
                 launch_environment = gdm_create_greeter_launch_environment (NULL,
                                                                             seat_id,
@@ -119,10 +116,9 @@ gdm_local_display_prepare (GdmDisplay *display)
                                                                             TRUE);
         } else {
                 launch_environment = gdm_create_initial_setup_launch_environment (NULL,
-                                                                                  seat_id,
-                                                                                  session_type,
-                                                                                  NULL,
-                                                                                  TRUE);
+                                                                                seat_id,
+                                                                                NULL,
+                                                                                TRUE);
         }
 
         g_object_set (self, "launch-environment", launch_environment, NULL);
@@ -132,10 +128,6 @@ out:
         g_free (seat_id);
         g_free (session_class);
         g_free (session_type);
-
-        if (failed) {
-                return FALSE;
-        }
         return GDM_DISPLAY_CLASS (gdm_local_display_parent_class)->prepare (display);
 }
 
