@@ -45,8 +45,10 @@
 #include "gdm-manager-glue.h"
 #include "gdm-display-store.h"
 #include "gdm-display-factory.h"
+#include "gdm-local-display.h"
 #include "gdm-local-display-factory.h"
 #include "gdm-session.h"
+#include "gdm-session-record.h"
 #include "gdm-xdmcp-display-factory.h"
 
 #define GDM_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GDM_TYPE_MANAGER, GdmManagerPrivate))
@@ -78,12 +80,15 @@ struct GdmManagerPrivate
         GCancellable           *cancellable;
 
         gboolean                started;
-        gboolean                wait_for_go;
         gboolean                show_local_greeter;
 
         GDBusProxy               *bus_proxy;
         GDBusConnection          *connection;
         GDBusObjectManagerServer *object_manager;
+
+#ifdef  WITH_PLYMOUTH
+        guint                     plymouth_is_running : 1;
+#endif
 };
 
 enum {
@@ -98,14 +103,20 @@ enum {
         LAST_SIGNAL
 };
 
+typedef enum {
+        SESSION_RECORD_LOGIN,
+        SESSION_RECORD_LOGOUT,
+        SESSION_RECORD_FAILED,
+} SessionRecord;
+
 static guint signals [LAST_SIGNAL] = { 0, };
 
 static void     gdm_manager_class_init  (GdmManagerClass *klass);
 static void     gdm_manager_init        (GdmManager      *manager);
 static void     gdm_manager_finalize    (GObject         *object);
-static void create_seed_session_for_display (GdmManager *manager,
-                                             GdmDisplay *display,
-                                             uid_t       allowed_user);
+static void     create_embryonic_user_session_for_display (GdmManager *manager,
+                                                           GdmDisplay *display,
+                                                           uid_t       allowed_user);
 static void     touch_ran_once_marker_file  (GdmManager *manager);
 
 static gpointer manager_object = NULL;
@@ -118,6 +129,72 @@ G_DEFINE_TYPE_WITH_CODE (GdmManager,
                          G_IMPLEMENT_INTERFACE (GDM_DBUS_TYPE_MANAGER,
                                                 manager_interface_init));
 
+#ifdef WITH_PLYMOUTH
+static gboolean
+plymouth_is_running (void)
+{
+        int      status;
+        gboolean res;
+        GError  *error;
+
+        error = NULL;
+        res = g_spawn_command_line_sync ("/bin/plymouth --ping",
+                                         NULL, NULL, &status, &error);
+        if (! res) {
+                g_debug ("Could not ping plymouth: %s", error->message);
+                g_error_free (error);
+                return FALSE;
+        }
+
+        return WIFEXITED (status) && WEXITSTATUS (status) == 0;
+}
+
+static void
+plymouth_prepare_for_transition (void)
+{
+        gboolean res;
+        GError  *error;
+
+        error = NULL;
+        res = g_spawn_command_line_sync ("/bin/plymouth deactivate",
+                                         NULL, NULL, NULL, &error);
+        if (! res) {
+                g_warning ("Could not deactivate plymouth: %s", error->message);
+                g_error_free (error);
+        }
+}
+
+static void
+plymouth_quit_with_transition (void)
+{
+        gboolean res;
+        GError  *error;
+
+        error = NULL;
+        res = g_spawn_command_line_sync ("/bin/plymouth quit --retain-splash",
+                                         NULL, NULL, NULL, &error);
+        if (! res) {
+                g_warning ("Could not quit plymouth: %s", error->message);
+                g_error_free (error);
+        }
+}
+
+static void
+plymouth_quit_without_transition (void)
+{
+        gboolean res;
+        GError  *error;
+
+        error = NULL;
+        res = g_spawn_command_line_sync ("/bin/plymouth quit",
+                                         NULL, NULL, NULL, &error);
+        if (! res) {
+                g_warning ("Could not quit plymouth: %s", error->message);
+                g_error_free (error);
+        }
+}
+#endif
+
 #ifdef WITH_SYSTEMD
 static char *
 get_session_id_for_pid_systemd (pid_t    pid,
@@ -906,13 +983,118 @@ get_display_for_user_session (GdmSession *session)
 }
 
 static GdmSession *
-get_seed_session_for_display (GdmDisplay *display)
+get_embryonic_user_session_for_display (GdmDisplay *display)
 {
         if (display == NULL) {
                 return NULL;
         }
 
-        return g_object_get_data (G_OBJECT (display), "gdm-seed-session");
+        return g_object_get_data (G_OBJECT (display), "gdm-embryonic-user-session");
+}
+
+static gboolean
+add_session_record (GdmManager    *manager,
+                    GdmSession    *session,
+                    GPid           pid,
+                    SessionRecord  record)
+{
+        const char *username;
+        char *display_name, *hostname, *display_device;
+        gboolean recorded = FALSE;
+
+        display_name = NULL;
+        username = NULL;
+        hostname = NULL;
+        display_device = NULL;
+
+        username = gdm_session_get_username (session);
+        g_object_get (G_OBJECT (session),
+                      "display-name", &display_name,
+                      "display-hostname", &hostname,
+                      "display-device", &display_device,
+                      NULL);
+
+        if (display_name == NULL) {
+                goto out;
+        }
+
+        switch (record) {
+            case SESSION_RECORD_LOGIN:
+                gdm_session_record_login (pid,
+                                          username,
+                                          hostname,
+                                          display_name,
+                                          display_device);
+                break;
+            case SESSION_RECORD_LOGOUT:
+                gdm_session_record_logout (pid,
+                                           username,
+                                           hostname,
+                                           display_name,
+                                           display_device);
+                break;
+            case SESSION_RECORD_FAILED:
+                gdm_session_record_failed (pid,
+                                           username,
+                                           hostname,
+                                           display_name,
+                                           display_device);
+                break;
+        }
+
+        recorded = TRUE;
+out:
+        g_free (display_name);
+        g_free (hostname);
+        g_free (display_device);
+
+        return recorded;
+}
+
+static gboolean
+gdm_manager_handle_register_display (GdmDBusManager        *manager,
+                                     GDBusMethodInvocation *invocation,
+                                     GVariant              *details)
+{
+        GdmManager      *self = GDM_MANAGER (manager);
+        const char      *sender;
+        GDBusConnection *connection;
+        GdmDisplay      *display = NULL;
+        GdmSession      *session;
+
+        g_debug ("GdmManager: trying to register new display");
+
+        sender = g_dbus_method_invocation_get_sender (invocation);
+        connection = g_dbus_method_invocation_get_connection (invocation);
+        get_display_and_details_for_bus_sender (self, connection, sender, &display, NULL, NULL, NULL, NULL, NULL, NULL);
+
+        if (display == NULL) {
+                g_dbus_method_invocation_return_error_literal (invocation,
+                                                               G_DBUS_ERROR,
+                                                               G_DBUS_ERROR_ACCESS_DENIED,
+                                                               _("No display available"));
+
+                return TRUE;
+        }
+
+        session = get_embryonic_user_session_for_display (display);
+
+        if (session != NULL) {
+                GPid pid;
+
+                pid = gdm_session_get_pid (session);
+
+                if (pid > 0) {
+                        add_session_record (self, session, pid, SESSION_RECORD_LOGIN);
+                }
+        }
+
+        g_object_set (G_OBJECT (display), "status", GDM_DISPLAY_MANAGED, NULL);
+
+        gdm_dbus_manager_complete_register_display (GDM_DBUS_MANAGER (manager),
+                                                    invocation);
+
+        return TRUE;
 }
 
 static gboolean
@@ -921,7 +1103,6 @@ gdm_manager_handle_open_session (GdmDBusManager        *manager,
 {
         GdmManager       *self = GDM_MANAGER (manager);
         const char       *sender;
-        GError           *error = NULL;
         GDBusConnection  *connection;
         GdmDisplay       *display = NULL;
         GdmSession       *session;
@@ -945,29 +1126,33 @@ gdm_manager_handle_open_session (GdmDBusManager        *manager,
                 return TRUE;
         }
 
-        session = get_seed_session_for_display (display);
+        session = get_embryonic_user_session_for_display (display);
 
         if (gdm_session_is_running (session)) {
-                error = g_error_new (G_DBUS_ERROR,
-                                     G_DBUS_ERROR_ACCESS_DENIED,
-                                     _("Can only be called before user is logged in"));
-                return FALSE;
+                g_dbus_method_invocation_return_error_literal (invocation,
+                                                               G_DBUS_ERROR,
+                                                               G_DBUS_ERROR_ACCESS_DENIED,
+                                                               _("Can only be called before user is logged in"));
+                return TRUE;
         }
 
         allowed_user = gdm_session_get_allowed_user (session);
 
         if (uid != allowed_user) {
-                error = g_error_new (G_DBUS_ERROR,
-                                     G_DBUS_ERROR_ACCESS_DENIED,
-                                     _("Caller not GDM"));
-                return FALSE;
+                g_dbus_method_invocation_return_error_literal (invocation,
+                                                               G_DBUS_ERROR,
+                                                               G_DBUS_ERROR_ACCESS_DENIED,
+                                                               _("Caller not GDM"));
+                return TRUE;
         }
 
         address = gdm_session_get_server_address (session);
 
         if (address == NULL) {
-                g_dbus_method_invocation_return_gerror (invocation, error);
-                g_error_free (error);
+                g_dbus_method_invocation_return_error_literal (invocation,
+                                                               G_DBUS_ERROR,
+                                                               G_DBUS_ERROR_ACCESS_DENIED,
+                                                               _("Unable to open private communication channel"));
                 return TRUE;
         }
 
@@ -1188,14 +1373,6 @@ gdm_manager_handle_open_reauthentication_channel (GdmDBusManager        *manager
         connection = g_dbus_method_invocation_get_connection (invocation);
         get_display_and_details_for_bus_sender (self, connection, sender, &display, &seat_id, &session_id, &pid, &uid, &is_login_screen, &is_remote);
 
-        if (is_login_screen) {
-                g_dbus_method_invocation_return_error_literal (invocation,
-                                                               G_DBUS_ERROR,
-                                                               G_DBUS_ERROR_ACCESS_DENIED,
-                                                               "Login screen not allow to open reauthentication channel");
-                return TRUE;
-        }
-
         if (session_id == NULL || pid == 0 || uid == (uid_t) -1) {
                 g_dbus_method_invocation_return_error_literal (invocation,
                                                                G_DBUS_ERROR,
@@ -1205,13 +1382,26 @@ gdm_manager_handle_open_reauthentication_channel (GdmDBusManager        *manager
                 return TRUE;
         }
 
-        session = get_seed_session_for_display (display);
+        if (is_login_screen) {
+                session = find_session_for_user_on_seat (self,
+                                                         username,
+                                                         seat_id,
+                                                         NULL);
+        } else {
+                session = get_embryonic_user_session_for_display (display);
+        }
 
         if (session != NULL && gdm_session_is_running (session)) {
                 gdm_session_start_reauthentication (session, pid, uid);
                 g_hash_table_insert (self->priv->open_reauthentication_requests,
                                      GINT_TO_POINTER (pid),
                                      invocation);
+        } else if (is_login_screen) {
+                g_dbus_method_invocation_return_error_literal (invocation,
+                                                               G_DBUS_ERROR,
+                                                               G_DBUS_ERROR_ACCESS_DENIED,
+                                                               "Login screen only allowed to open reauthentication channels for running sessions");
+                return TRUE;
         } else {
                 char *address;
                 address = open_temporary_reauthentication_channel (self,
@@ -1232,18 +1422,42 @@ gdm_manager_handle_open_reauthentication_channel (GdmDBusManager        *manager
 static void
 manager_interface_init (GdmDBusManagerIface *interface)
 {
+        interface->handle_register_display = gdm_manager_handle_register_display;
         interface->handle_open_session = gdm_manager_handle_open_session;
         interface->handle_open_reauthentication_channel = gdm_manager_handle_open_reauthentication_channel;
 }
 
+static gboolean
+display_should_autologin (GdmManager *manager,
+                          GdmDisplay *display)
+{
+        gboolean enabled = FALSE;
+        int delay = 0;
+
+        if (g_file_test (GDM_RAN_ONCE_MARKER_FILE, G_FILE_TEST_EXISTS)) {
+                return FALSE;
+        }
+
+        gdm_display_get_timed_login_details (display, &enabled, NULL, &delay);
+        return enabled && delay == 0;
+}
+
 static void
 set_up_greeter_session (GdmManager *manager,
                         GdmDisplay *display)
 {
         char *allowed_user;
         struct passwd *passwd_entry;
+        gboolean will_autologin;
 
-        gdm_display_set_up_greeter_session (display, &allowed_user);
+        will_autologin = display_should_autologin (manager, display);
+
+        if (!will_autologin) {
+                 gdm_display_set_up_greeter_session (display, &allowed_user);
+        } else {
+                 g_object_set (G_OBJECT (display), "session-class", "user", NULL);
+                 allowed_user = g_strdup ("root");
+        }
 
         if (!gdm_get_pwent_for_name (allowed_user, &passwd_entry)) {
                 g_warning ("GdmManager: couldn't look up username %s",
@@ -1253,10 +1467,13 @@ set_up_greeter_session (GdmManager *manager,
                 return;
         }
 
-        create_seed_session_for_display (manager, display, passwd_entry->pw_uid);
+        create_embryonic_user_session_for_display (manager, display, passwd_entry->pw_uid);
         g_free (allowed_user);
 
-        gdm_display_start_greeter_session (display);
+        if (!will_autologin) {
+                gdm_display_start_greeter_session (display);
+        }
+
         touch_ran_once_marker_file (manager);
 }
 
@@ -1266,16 +1483,50 @@ on_display_status_changed (GdmDisplay *display,
                            GdmManager *manager)
 {
         int         status;
+        int         display_number = -1;
+#ifdef WITH_PLYMOUTH
+        gboolean    display_is_local = FALSE;
+        gboolean    quit_plymouth = FALSE;
+
+        g_object_get (display,
+                      "is-local", &display_is_local,
+                      "x11-display-number", &display_number,
+                      NULL);
+        quit_plymouth = display_is_local && manager->priv->plymouth_is_running;
+#endif
 
         status = gdm_display_get_status (display);
 
         switch (status) {
+                case GDM_DISPLAY_PREPARED:
                 case GDM_DISPLAY_MANAGED:
-                        set_up_greeter_session (manager, display);
+#ifdef WITH_PLYMOUTH
+                        if (quit_plymouth) {
+                                plymouth_quit_with_transition ();
+                                manager->priv->plymouth_is_running = FALSE;
+                        }
+#endif
+                        if ((display_number == -1 && status == GDM_DISPLAY_PREPARED) ||
+                            (display_number != -1 && status == GDM_DISPLAY_MANAGED)) {
+                                char *session_class;
+
+                                g_object_get (display,
+                                              "session-class", &session_class,
+                                              NULL);
+                                if (g_strcmp0 (session_class, "greeter") == 0) {
+                                        set_up_greeter_session (manager, display);
+                                }
+                        }
                         break;
                 case GDM_DISPLAY_FAILED:
                 case GDM_DISPLAY_UNMANAGED:
                 case GDM_DISPLAY_FINISHED:
+#ifdef WITH_PLYMOUTH
+                        if (quit_plymouth) {
+                                plymouth_quit_without_transition ();
+                                manager->priv->plymouth_is_running = FALSE;
+                        }
+#endif
                         break;
                 default:
                         break;
@@ -1330,21 +1581,26 @@ start_user_session (GdmManager *manager,
         if (display != NULL) {
                 char *auth_file;
                 const char *username;
+                gboolean is_connected = FALSE;
 
-                auth_file = NULL;
-                username = gdm_session_get_username (operation->session);
-                gdm_display_add_user_authorization (display,
-                                                    username,
-                                                    &auth_file,
-                                                    NULL);
+                g_object_get (G_OBJECT (display), "is-connected", &is_connected, NULL);
 
-                g_assert (auth_file != NULL);
+                if (is_connected) {
+                        auth_file = NULL;
+                        username = gdm_session_get_username (operation->session);
+                        gdm_display_add_user_authorization (display,
+                                                            username,
+                                                            &auth_file,
+                                                            NULL);
 
-                g_object_set (operation->session,
-                              "user-x11-authority-file", auth_file,
-                              NULL);
+                        g_assert (auth_file != NULL);
 
-                g_free (auth_file);
+                        g_object_set (operation->session,
+                                      "user-x11-authority-file", auth_file,
+                                      NULL);
+
+                        g_free (auth_file);
+                }
         }
 
         gdm_session_start_session (operation->session,
@@ -1352,6 +1608,24 @@ start_user_session (GdmManager *manager,
         destroy_start_user_session_operation (operation);
 }
 
+static void
+create_display_for_user_session (GdmManager *self,
+                                 GdmSession *session,
+                                 const char *session_id)
+{
+        GdmDisplay *display;
+
+        display = gdm_local_display_new ();
+
+        g_object_set (G_OBJECT (display),
+                      "session-class", "user",
+                      "session-id", session_id,
+                      NULL);
+        gdm_display_store_add (self->priv->display_store,
+                               display);
+        g_object_set_data (G_OBJECT (session), "gdm-display", display);
+}
+
 static gboolean
 on_start_user_session (StartUserSessionOperation *operation)
 {
@@ -1383,20 +1657,29 @@ on_start_user_session (StartUserSessionOperation *operation)
         if (gdm_session_get_display_mode (operation->session) == GDM_SESSION_DISPLAY_MODE_REUSE_VT) {
                 /* In this case, the greeter's display is morphing into
                  * the user session display. Kill the greeter on this session
-                 * and let the seed session follow the same display. */
+                 * and let the user session follow the same display. */
                 gdm_display_stop_greeter_session (display);
+                g_object_set (G_OBJECT (display), "session-class", "user", NULL);
         } else {
+                const char *session_id;
                 uid_t allowed_uid;
 
                 g_debug ("GdmManager: session has its display server, reusing our server for another login screen");
 
-                /* The seed session is going to follow the session worker
+                /* The user session is going to follow the session worker
                  * into the new display. Untie it from this display and
-                 * create a new seed session for us. */
+                 * create a new embryonic session for a future user login. */
                 allowed_uid = gdm_session_get_allowed_user (operation->session);
-                g_object_set_data (G_OBJECT (display), "gdm-seed-session", NULL);
+                g_object_set_data (G_OBJECT (display), "gdm-embryonic-user-session", NULL);
                 g_object_set_data (G_OBJECT (operation->session), "gdm-display", NULL);
-                create_seed_session_for_display (operation->manager, display, allowed_uid);
+                create_embryonic_user_session_for_display (operation->manager, display, allowed_uid);
+
+                /* Give the user session a new display object for bookkeeping purposes */
+                session_id = gdm_session_get_conversation_session_id (operation->session,
+                                                                      operation->service_name);
+                create_display_for_user_session (operation->manager,
+                                                 operation->session,
+                                                 session_id);
         }
 
         start_user_session (operation->manager, operation);
@@ -1438,10 +1721,19 @@ start_user_session_if_ready (GdmManager *manager,
 }
 
 static void
-on_session_opened (GdmSession       *session,
-                   const char       *service_name,
-                   const char       *session_id,
-                   GdmManager       *manager)
+on_session_authentication_failed (GdmSession *session,
+                                  const char *service_name,
+                                  GPid        conversation_pid,
+                                  GdmManager *manager)
+{
+        add_session_record (manager, session, conversation_pid, SESSION_RECORD_FAILED);
+}
+
+static void
+on_user_session_opened (GdmSession       *session,
+                        const char       *service_name,
+                        const char       *session_id,
+                        GdmManager       *manager)
 {
         manager->priv->user_sessions = g_list_append (manager->priv->user_sessions,
                                                       g_object_ref (session));
@@ -1456,26 +1748,13 @@ on_session_opened (GdmSession       *session,
 }
 
 static void
-on_session_started (GdmSession      *session,
-                    const char      *service_name,
-                    GPid             pid,
-                    GdmManager      *manager)
+on_user_session_started (GdmSession      *session,
+                         const char      *service_name,
+                         GPid             pid,
+                         GdmManager      *manager)
 {
-        GdmDisplay *display;
-
         g_debug ("GdmManager: session started %d", pid);
-
-        display = get_display_for_user_session (session);
-
-        if (display != NULL) {
-                GdmSlave *slave;
-                const char *session_id;
-
-                slave = gdm_display_get_slave (display);
-
-                session_id = gdm_session_get_session_id (session);
-                g_object_set (GDM_SLAVE (slave), "session-id", session_id, NULL);
-        }
+        add_session_record (manager, session, pid, SESSION_RECORD_LOGIN);
 }
 
 static void
@@ -1506,7 +1785,15 @@ on_user_session_exited (GdmSession *session,
                         int         code,
                         GdmManager *manager)
 {
+        GPid pid;
+
         g_debug ("GdmManager: session exited with status %d", code);
+        pid = gdm_session_get_pid (session);
+
+        if (pid > 0) {
+                add_session_record (manager, session, pid, SESSION_RECORD_LOGOUT);
+        }
+
         remove_user_session (manager, session);
 }
 
@@ -1635,7 +1922,7 @@ on_session_client_connected (GdmSession      *session,
         g_object_get (display, "is-local", &display_is_local, NULL);
 
         enabled = FALSE;
-        gdm_display_get_timed_login_details (display, &enabled, &username, &delay, NULL);
+        gdm_display_get_timed_login_details (display, &enabled, &username, &delay);
 
         if (! enabled) {
                 return;
@@ -1755,7 +2042,7 @@ on_session_conversation_started (GdmSession *session,
         }
 
         enabled = FALSE;
-        gdm_display_get_timed_login_details (display, &enabled, &username, &delay, NULL);
+        gdm_display_get_timed_login_details (display, &enabled, &username, &delay);
         if (! enabled) {
                 return;
         }
@@ -1811,7 +2098,7 @@ start_autologin_conversation_if_necessary (GdmManager *manager,
         gboolean enabled;
         int delay = 0;
 
-        gdm_display_get_timed_login_details (display, &enabled, NULL, &delay, NULL);
+        gdm_display_get_timed_login_details (display, &enabled, NULL, &delay);
 
         if (delay == 0 && g_file_test (GDM_RAN_ONCE_MARKER_FILE, G_FILE_TEST_EXISTS)) {
                 return;
@@ -1843,7 +2130,14 @@ touch_ran_once_marker_file (GdmManager *manager)
 }
 
 static void
-create_seed_session_for_display (GdmManager *manager,
+clean_embryonic_user_session (GdmSession *session)
+{
+        g_object_set_data (G_OBJECT (session), "gdm-display", NULL);
+        g_object_unref (session);
+}
+
+static void
+create_embryonic_user_session_for_display (GdmManager *manager,
                                  GdmDisplay *display,
                                  uid_t       allowed_user)
 {
@@ -1912,13 +2206,17 @@ create_seed_session_for_display (GdmManager *manager,
                           "conversation-stopped",
                           G_CALLBACK (on_session_conversation_stopped),
                           manager);
+        g_signal_connect (session,
+                          "authentication-failed",
+                          G_CALLBACK (on_session_authentication_failed),
+                          manager);
         g_signal_connect (session,
                           "session-opened",
-                          G_CALLBACK (on_session_opened),
+                          G_CALLBACK (on_user_session_opened),
                           manager);
         g_signal_connect (session,
                           "session-started",
-                          G_CALLBACK (on_session_started),
+                          G_CALLBACK (on_user_session_started),
                           manager);
         g_signal_connect (session,
                           "session-exited",
@@ -1929,7 +2227,11 @@ create_seed_session_for_display (GdmManager *manager,
                           G_CALLBACK (on_user_session_died),
                           manager);
         g_object_set_data (G_OBJECT (session), "gdm-display", display);
-        g_object_set_data_full (G_OBJECT (display), "gdm-seed-session", g_object_ref (session), (GDestroyNotify) g_object_unref);
+        g_object_set_data_full (G_OBJECT (display),
+                                "gdm-embryonic-user-session",
+                                g_object_ref (session),
+                                (GDestroyNotify)
+                                clean_embryonic_user_session);
 
         start_autologin_conversation_if_necessary (manager, display, session);
 }
@@ -2025,13 +2327,20 @@ gdm_manager_start (GdmManager *manager)
 {
         g_debug ("GdmManager: GDM starting to manage displays");
 
-        if (! manager->priv->wait_for_go && (!manager->priv->xdmcp_enabled || manager->priv->show_local_greeter)) {
+#ifdef WITH_PLYMOUTH
+        manager->priv->plymouth_is_running = plymouth_is_running ();
+
+        if (manager->priv->plymouth_is_running) {
+                plymouth_prepare_for_transition ();
+        }
+#endif
+        if (!manager->priv->xdmcp_enabled || manager->priv->show_local_greeter) {
                 gdm_display_factory_start (GDM_DISPLAY_FACTORY (manager->priv->local_factory));
         }
 
 #ifdef HAVE_LIBXDMCP
         /* Accept remote connections */
-        if (manager->priv->xdmcp_enabled && ! manager->priv->wait_for_go) {
+        if (manager->priv->xdmcp_enabled) {
                 if (manager->priv->xdmcp_factory != NULL) {
                         g_debug ("GdmManager: Accepting XDMCP connections...");
                         gdm_display_factory_start (GDM_DISPLAY_FACTORY (manager->priv->xdmcp_factory));
@@ -2042,29 +2351,6 @@ gdm_manager_start (GdmManager *manager)
         manager->priv->started = TRUE;
 }
 
-void
-gdm_manager_set_wait_for_go (GdmManager *manager,
-                             gboolean    wait_for_go)
-{
-        if (manager->priv->wait_for_go != wait_for_go) {
-                manager->priv->wait_for_go = wait_for_go;
-
-                if (! wait_for_go) {
-                        /* we got a go */
-                        if (!manager->priv->xdmcp_enabled || manager->priv->show_local_greeter) {
-                                gdm_display_factory_start (GDM_DISPLAY_FACTORY (manager->priv->local_factory));
-                        }
-
-#ifdef HAVE_LIBXDMCP
-                        if (manager->priv->xdmcp_enabled && manager->priv->xdmcp_factory != NULL) {
-                                g_debug ("GdmManager: Accepting XDMCP connections...");
-                                gdm_display_factory_start (GDM_DISPLAY_FACTORY (manager->priv->xdmcp_factory));
-                        }
-#endif
-                }
-        }
-}
-
 static gboolean
 register_manager (GdmManager *manager)
 {
@@ -2292,8 +2578,10 @@ finish_display (const char *id,
                 GdmDisplay *display,
                 GdmManager *manager)
 {
-        if (gdm_display_get_status (display) == GDM_DISPLAY_MANAGED)
-                gdm_display_unmanage (display);
+        if (gdm_display_get_status (display) != GDM_DISPLAY_MANAGED)
+                return;
+
+        gdm_display_unmanage (display);
         gdm_display_finish (display);
 }
 
