@@ -37,17 +37,14 @@
 #include <gio/gunixinputstream.h>
 
 #define BUS_ADDRESS_FILENO (STDERR_FILENO + 1)
+#define DBUS_DAEMON BINDIR "/dbus-daemon"
 
 typedef struct
 {
         GdmSettings  *settings;
         GCancellable *cancellable;
-
-        GSubprocess     *bus_subprocess;
-        GDBusConnection *bus_connection;
-        char            *bus_address;
-
-        char           **environment;
+        GSubprocess  *bus_subprocess;
+        char         *bus_address;
 
         GSubprocess  *session_subprocess;
         char         *session_command;
@@ -84,8 +81,8 @@ on_bus_finished (GSubprocess  *subprocess,
                 g_debug ("message bus was killed with status %d", signal_number);
         }
 
-        g_clear_object (&state->bus_subprocess);
 out:
+        g_clear_object (&state->bus_subprocess);
         g_main_loop_quit (state->main_loop);
 }
 
@@ -93,15 +90,14 @@ static gboolean
 spawn_bus (State        *state,
            GCancellable *cancellable)
 {
-        GDBusConnection     *bus_connection = NULL;
-        GPtrArray           *arguments = NULL;
-        GSubprocessLauncher *launcher = NULL;
-        GSubprocess         *subprocess = NULL;
-        GInputStream        *input_stream = NULL;
-        GDataInputStream    *data_stream = NULL;
+        GPtrArray           *arguments;
+        GSubprocessLauncher *launcher;
+        GSubprocess         *subprocess;
+        GInputStream        *input_stream;
+        GDataInputStream    *data_stream;
         GError              *error = NULL;
-        char                *bus_address_fd_string = NULL;
-        char                *bus_address = NULL;
+        char                *bus_address_fd_string;
+        char                *bus_address;
         gsize                bus_address_size;
 
         gboolean  is_running = FALSE;
@@ -110,16 +106,6 @@ spawn_bus (State        *state,
 
         g_debug ("Running session message bus");
 
-        bus_connection = g_bus_get_sync (G_BUS_TYPE_SESSION,
-                                         cancellable,
-                                         NULL);
-
-        if (bus_connection != NULL) {
-                g_debug ("session message bus already running, not starting another one");
-                state->bus_connection = bus_connection;
-                return TRUE;
-        }
-
         ret = g_unix_open_pipe (pipe_fds, FD_CLOEXEC, &error);
 
         if (!ret) {
@@ -133,7 +119,7 @@ spawn_bus (State        *state,
 
         bus_address_fd_string = g_strdup_printf ("%d", BUS_ADDRESS_FILENO);
 
-        g_ptr_array_add (arguments, "dbus-daemon");
+        g_ptr_array_add (arguments, DBUS_DAEMON);
 
         g_ptr_array_add (arguments, "--print-address");
         g_ptr_array_add (arguments, bus_address_fd_string);
@@ -148,7 +134,7 @@ spawn_bus (State        *state,
         g_ptr_array_free (arguments, TRUE);
 
         if (subprocess == NULL) {
-                g_debug ("could not start dbus-daemon: %s", error->message);
+                g_debug ("could not start X server: %s", error->message);
                 goto out;
         }
 
@@ -181,20 +167,6 @@ spawn_bus (State        *state,
                                  on_bus_finished,
                                  state);
 
-        bus_connection = g_dbus_connection_new_for_address_sync (state->bus_address,
-                                                                 G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT |
-                                                                 G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION,
-                                                                 NULL,
-                                                                 cancellable,
-                                                                 &error);
-
-        if (bus_connection == NULL) {
-                g_debug ("could not open connection to session bus: %s",
-                         error->message);
-                goto out;
-        }
-
-        state->bus_connection = bus_connection;
         is_running = TRUE;
 out:
         g_clear_object (&data_stream);
@@ -205,38 +177,6 @@ out:
         return is_running;
 }
 
-static gboolean
-import_environment (State        *state,
-                    GCancellable *cancellable)
-{
-        g_autoptr(GVariant) reply = NULL;
-        g_autoptr(GVariant) environment_variant = NULL;
-        g_autoptr(GError)   error = NULL;
-
-        reply = g_dbus_connection_call_sync (state->bus_connection,
-                                             "org.freedesktop.systemd1",
-                                             "/org/freedesktop/systemd1",
-                                             "org.freedesktop.DBus.Properties",
-                                             "Get",
-                                             g_variant_new ("(ss)",
-                                                            "org.freedesktop.systemd1.Manager",
-                                                            "Environment"),
-                                             NULL,
-                                             G_DBUS_CALL_FLAGS_NONE,
-                                             -1, cancellable, &error);
-
-        if (reply == NULL) {
-                g_debug ("could not fetch environment: %s", error->message);
-                return FALSE;
-        }
-
-        g_variant_get (reply, "(v)", &environment_variant);
-
-        state->environment = g_variant_dup_strv (environment_variant, NULL);
-
-        return TRUE;
-}
-
 static void
 on_session_finished (GSubprocess  *subprocess,
                      GAsyncResult *result,
@@ -265,8 +205,8 @@ on_session_finished (GSubprocess  *subprocess,
                 g_debug ("session was killed with status %d", signal_number);
         }
 
-        g_clear_object (&state->session_subprocess);
 out:
+        g_clear_object (&state->session_subprocess);
         g_main_loop_quit (state->main_loop);
 }
 
@@ -274,18 +214,12 @@ static gboolean
 spawn_session (State        *state,
                GCancellable *cancellable)
 {
-        GSubprocessLauncher *launcher = NULL;
+        GSubprocessLauncher *launcher;
         GSubprocess         *subprocess = NULL;
         GError              *error = NULL;
         gboolean             is_running = FALSE;
         int                  ret;
-        char               **argv = NULL;
-        static const char  *session_variables[] = { "DISPLAY",
-                                                    "XAUTHORITY",
-                                                    "WAYLAND_DISPLAY",
-                                                    "WAYLAND_SOCKET",
-                                                    "GNOME_SHELL_SESSION_MODE",
-                                                    NULL };
+        char               **argv;
 
         g_debug ("Running wayland session");
 
@@ -300,39 +234,7 @@ spawn_session (State        *state,
         }
 
         launcher = g_subprocess_launcher_new (G_SUBPROCESS_FLAGS_NONE);
-
-        if (state->environment != NULL) {
-                size_t i;
-
-                for (i = 0; state->environment[i] != NULL; i++) {
-                        g_auto(GStrv) environment_entry = NULL;
-
-                        if (state->environment[i][0] == '\0') {
-                                continue;
-                        }
-
-                        environment_entry = g_strsplit (state->environment[i], "=", 2);
-
-                        if (environment_entry[0] == NULL || environment_entry[1] == NULL) {
-                                continue;
-                        }
-
-                        g_subprocess_launcher_setenv (launcher, environment_entry[0], environment_entry[1], FALSE);
-                }
-
-                /* Don't allow session specific environment variables from earlier sessions to
-                 * leak through */
-                for (i = 0; session_variables[i] != NULL; i++) {
-                        if (g_getenv (session_variables[i]) == NULL) {
-                                g_subprocess_launcher_unsetenv (launcher, session_variables[i]);
-                        }
-                }
-        }
-
-        if (state->bus_address != NULL) {
-                g_subprocess_launcher_setenv (launcher, "DBUS_SESSION_BUS_ADDRESS", state->bus_address, TRUE);
-        }
-
+        g_subprocess_launcher_setenv (launcher, "DBUS_SESSION_BUS_ADDRESS", state->bus_address, TRUE);
         subprocess = g_subprocess_launcher_spawnv (launcher,
                                                    (const char * const *) argv,
                                                    &error);
@@ -385,7 +287,7 @@ static gboolean
 register_display (State        *state,
                   GCancellable *cancellable)
 {
-        GdmDBusManager  *manager = NULL;
+        GdmDBusManager  *manager;
         GError          *error = NULL;
         gboolean         registered = FALSE;
         GVariantBuilder  details;
@@ -435,9 +337,7 @@ clear_state (State **out_state)
         State *state = *out_state;
 
         g_clear_object (&state->cancellable);
-        g_clear_object (&state->bus_connection);
         g_clear_object (&state->session_subprocess);
-        g_clear_pointer (&state->environment, g_strfreev);
         g_clear_pointer (&state->main_loop, g_main_loop_unref);
         *out_state = NULL;
 }
@@ -458,8 +358,8 @@ int
 main (int    argc,
       char **argv)
 {
-        State           *state = NULL;
-        GOptionContext  *context = NULL;
+        State           *state;
+        GOptionContext  *context;
         static char    **args = NULL;
         gboolean         debug = FALSE;
         gboolean         ret;
@@ -495,7 +395,7 @@ main (int    argc,
         ret = gdm_settings_direct_init (state->settings, DATADIR "/gdm/gdm.schemas", "/");
 
         if (!ret) {
-                g_printerr ("Unable to initialize settings\n");
+                g_printerr ("Unable to initialize settings");
                 exit_status = EX_DATAERR;
                 goto out;
         }
@@ -513,17 +413,15 @@ main (int    argc,
         ret = spawn_bus (state, state->cancellable);
 
         if (!ret) {
-                g_printerr ("Unable to run session message bus\n");
+                g_printerr ("Unable to run session message bus");
                 exit_status = EX_SOFTWARE;
                 goto out;
         }
 
-        import_environment (state, state->cancellable);
-
         ret = spawn_session (state, state->cancellable);
 
         if (!ret) {
-                g_printerr ("Unable to run session\n");
+                g_printerr ("Unable to run session");
                 exit_status = EX_SOFTWARE;
                 goto out;
         }
@@ -531,7 +429,7 @@ main (int    argc,
         ret = register_display (state, state->cancellable);
 
         if (!ret) {
-                g_printerr ("Unable to register display with display manager\n");
+                g_printerr ("Unable to register display with display manager");
                 exit_status = EX_SOFTWARE;
                 goto out;
         }
@@ -545,11 +443,9 @@ main (int    argc,
         }
 
 out:
-        if (state != NULL) {
-                signal_subprocesses (state);
-                wait_on_subprocesses (state);
-                clear_state (&state);
-        }
+        signal_subprocesses (state);
+        wait_on_subprocesses (state);
+        clear_state (&state);
 
         return exit_status;
 }
