@@ -40,6 +40,8 @@
 #define DISPLAY_FILENO (STDERR_FILENO + 1)
 #define BUS_ADDRESS_FILENO (DISPLAY_FILENO + 1)
 
+#define DBUS_DAEMON BINDIR "/dbus-daemon"
+
 typedef struct
 {
         GdmSettings  *settings;
@@ -49,19 +51,13 @@ typedef struct
         char         *auth_file;
         char         *display_name;
 
-        GSubprocess     *bus_subprocess;
-        GDBusConnection *bus_connection;
-        GdmDBusManager  *display_manager_proxy;
-        char            *bus_address;
-
-        char           **environment;
+        GSubprocess  *bus_subprocess;
+        char         *bus_address;
 
         GSubprocess  *session_subprocess;
         char         *session_command;
         int           session_exit_status;
 
-        guint         register_session_id;
-
         GMainLoop    *main_loop;
 
         guint32       debug_enabled : 1;
@@ -189,8 +185,8 @@ on_x_server_finished (GSubprocess  *subprocess,
                 g_debug ("X server was killed with status %d", signal_number);
         }
 
-        g_clear_object (&state->x_subprocess);
 out:
+        g_clear_object (&state->x_subprocess);
         g_main_loop_quit (state->main_loop);
 }
 
@@ -199,11 +195,11 @@ spawn_x_server (State        *state,
                 gboolean      allow_remote_connections,
                 GCancellable *cancellable)
 {
-        GPtrArray           *arguments = NULL;
-        GSubprocessLauncher *launcher = NULL;
-        GSubprocess         *subprocess = NULL;
-        GInputStream        *input_stream = NULL;
-        GDataInputStream    *data_stream = NULL;
+        GPtrArray           *arguments;
+        GSubprocessLauncher *launcher;
+        GSubprocess         *subprocess;
+        GInputStream        *input_stream;
+        GDataInputStream    *data_stream;
         GError              *error = NULL;
 
         char     *auth_file;
@@ -228,7 +224,7 @@ spawn_x_server (State        *state,
 
         arguments = g_ptr_array_new ();
         launcher = g_subprocess_launcher_new (G_SUBPROCESS_FLAGS_STDIN_INHERIT);
-        g_subprocess_launcher_setenv (launcher, "XORG_RUN_AS_USER_OK", "1", TRUE);
+        g_subprocess_launcher_setenv (launcher, "DBUS_SESSION_BUS_ADDRESS", state->bus_address, TRUE);
         g_subprocess_launcher_take_fd (launcher, pipe_fds[1], DISPLAY_FILENO);
 
         if (g_getenv ("XDG_VTNR") != NULL) {
@@ -255,29 +251,16 @@ spawn_x_server (State        *state,
         g_ptr_array_add (arguments, "-auth");
         g_ptr_array_add (arguments, auth_file);
 
-        /* If we were compiled with Xserver >= 1.17 we need to specify
-         * '-listen tcp' as the X server doesn't listen on tcp sockets
-         * by default anymore. In older versions we need to pass
-         * -nolisten tcp to disable listening on tcp sockets.
-         */
-#ifdef HAVE_XSERVER_THAT_DEFAULTS_TO_LOCAL_ONLY
-        if (allow_remote_connections) {
-                g_ptr_array_add (arguments, "-listen");
-                g_ptr_array_add (arguments, "tcp");
-        }
-#else
         if (!allow_remote_connections) {
                 g_ptr_array_add (arguments, "-nolisten");
                 g_ptr_array_add (arguments, "tcp");
         }
-#endif
 
         g_ptr_array_add (arguments, "-background");
         g_ptr_array_add (arguments, "none");
 
         g_ptr_array_add (arguments, "-noreset");
         g_ptr_array_add (arguments, "-keeptty");
-        g_ptr_array_add (arguments, "-novtswitch");
 
         g_ptr_array_add (arguments, "-verbose");
         if (state->debug_enabled) {
@@ -371,63 +354,23 @@ on_bus_finished (GSubprocess  *subprocess,
                 g_debug ("message bus was killed with status %d", signal_number);
         }
 
+out:
         g_clear_object (&state->bus_subprocess);
-out:
         g_main_loop_quit (state->main_loop);
 }
 
-static gboolean
-update_bus_environment (State        *state,
-                        GCancellable *cancellable)
-{
-        GVariantBuilder      builder;
-        GVariant            *reply = NULL;
-        GError              *error = NULL;
-        gboolean             environment_updated = FALSE;
-
-        g_variant_builder_init (&builder, G_VARIANT_TYPE ("a{ss}"));
-        g_variant_builder_add (&builder, "{ss}", "DISPLAY", state->display_name);
-        g_variant_builder_add (&builder, "{ss}", "XAUTHORITY", state->auth_file);
-
-        reply = g_dbus_connection_call_sync (state->bus_connection,
-                                             "org.freedesktop.DBus",
-                                             "/org/freedesktop/DBus",
-                                             "org.freedesktop.DBus",
-                                             "UpdateActivationEnvironment",
-                                             g_variant_new ("(@a{ss})",
-                                                            g_variant_builder_end (&builder)),
-                                             NULL,
-                                             G_DBUS_CALL_FLAGS_NONE,
-                                             -1, cancellable, &error);
-
-        if (reply == NULL) {
-                g_debug ("could not update activation environment: %s", error->message);
-                goto out;
-        }
-
-        g_variant_unref (reply);
-
-        environment_updated = TRUE;
-
-out:
-        g_clear_error (&error);
-
-        return environment_updated;
-}
-
 static gboolean
 spawn_bus (State        *state,
            GCancellable *cancellable)
 {
-        GDBusConnection     *bus_connection = NULL;
-        GPtrArray           *arguments = NULL;
-        GSubprocessLauncher *launcher = NULL;
-        GSubprocess         *subprocess = NULL;
-        GInputStream        *input_stream = NULL;
-        GDataInputStream    *data_stream = NULL;
+        GPtrArray           *arguments;
+        GSubprocessLauncher *launcher;
+        GSubprocess         *subprocess;
+        GInputStream        *input_stream;
+        GDataInputStream    *data_stream;
         GError              *error = NULL;
         char                *bus_address_fd_string;
-        char                *bus_address = NULL;
+        char                *bus_address;
         gsize                bus_address_size;
 
         gboolean  is_running = FALSE;
@@ -436,16 +379,6 @@ spawn_bus (State        *state,
 
         g_debug ("Running session message bus");
 
-        bus_connection = g_bus_get_sync (G_BUS_TYPE_SESSION,
-                                         cancellable,
-                                         NULL);
-
-        if (bus_connection != NULL) {
-                g_debug ("session message bus already running, not starting another one");
-                state->bus_connection = bus_connection;
-                return TRUE;
-        }
-
         ret = g_unix_open_pipe (pipe_fds, FD_CLOEXEC, &error);
 
         if (!ret) {
@@ -455,12 +388,11 @@ spawn_bus (State        *state,
 
         arguments = g_ptr_array_new ();
         launcher = g_subprocess_launcher_new (G_SUBPROCESS_FLAGS_NONE);
-
         g_subprocess_launcher_take_fd (launcher, pipe_fds[1], BUS_ADDRESS_FILENO);
 
         bus_address_fd_string = g_strdup_printf ("%d", BUS_ADDRESS_FILENO);
 
-        g_ptr_array_add (arguments, "dbus-daemon");
+        g_ptr_array_add (arguments, DBUS_DAEMON);
 
         g_ptr_array_add (arguments, "--print-address");
         g_ptr_array_add (arguments, bus_address_fd_string);
@@ -475,7 +407,7 @@ spawn_bus (State        *state,
         g_ptr_array_free (arguments, TRUE);
 
         if (subprocess == NULL) {
-                g_debug ("could not start dbus-daemon: %s", error->message);
+                g_debug ("could not start X server: %s", error->message);
                 goto out;
         }
 
@@ -508,22 +440,6 @@ spawn_bus (State        *state,
                                  on_bus_finished,
                                  state);
 
-
-        bus_connection = g_dbus_connection_new_for_address_sync (state->bus_address,
-                                                                 G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT |
-                                                                 G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION,
-                                                                 NULL,
-                                                                 cancellable,
-                                                                 &error);
-
-        if (bus_connection == NULL) {
-                g_debug ("could not open connection to session bus: %s",
-                         error->message);
-                goto out;
-        }
-
-        state->bus_connection = bus_connection;
-
         is_running = TRUE;
 out:
         g_clear_object (&data_stream);
@@ -534,37 +450,6 @@ out:
         return is_running;
 }
 
-static gboolean
-import_environment (State        *state,
-                    GCancellable *cancellable)
-{
-        g_autoptr(GVariant) reply = NULL;
-        g_autoptr(GVariant) environment_variant = NULL;
-        g_autoptr(GError)   error = NULL;
-
-        reply = g_dbus_connection_call_sync (state->bus_connection,
-                                             "org.freedesktop.systemd1",
-                                             "/org/freedesktop/systemd1",
-                                             "org.freedesktop.DBus.Properties",
-                                             "Get",
-                                             g_variant_new ("(ss)",
-                                                            "org.freedesktop.systemd1.Manager",
-                                                            "Environment"),
-                                             NULL,
-                                             G_DBUS_CALL_FLAGS_NONE,
-                                             -1, cancellable, &error);
-
-        if (reply == NULL) {
-                g_debug ("could not fetch environment: %s", error->message);
-                return FALSE;
-        }
-
-        g_variant_get (reply, "(v)", &environment_variant);
-
-        state->environment = g_variant_dup_strv (environment_variant, NULL);
-
-        return TRUE;
-}
 
 static void
 on_session_finished (GSubprocess  *subprocess,
@@ -594,8 +479,8 @@ on_session_finished (GSubprocess  *subprocess,
                 g_debug ("session was killed with status %d", signal_number);
         }
 
-        g_clear_object (&state->session_subprocess);
 out:
+        g_clear_object (&state->session_subprocess);
         g_main_loop_quit (state->main_loop);
 }
 
@@ -604,61 +489,23 @@ spawn_session (State        *state,
                gboolean      run_script,
                GCancellable *cancellable)
 {
-        GSubprocessLauncher *launcher = NULL;
-        GSubprocess         *subprocess = NULL;
+        GSubprocessLauncher *launcher;
+        GSubprocess         *subprocess;
         GError              *error = NULL;
         gboolean             is_running = FALSE;
         const char          *vt;
-        static const char   *session_variables[] = { "DISPLAY",
-                                                     "XAUTHORITY",
-                                                     "WAYLAND_DISPLAY",
-                                                     "WAYLAND_SOCKET",
-                                                     "GNOME_SHELL_SESSION_MODE",
-                                                     NULL };
 
         g_debug ("Running X session");
 
         launcher = g_subprocess_launcher_new (G_SUBPROCESS_FLAGS_NONE);
 
-        if (state->environment != NULL) {
-                size_t i;
-
-                for (i = 0; state->environment[i] != NULL; i++) {
-                        g_auto(GStrv) environment_entry = NULL;
-
-                        if (state->environment[i][0] == '\0') {
-                                continue;
-                        }
-
-                        environment_entry = g_strsplit (state->environment[i], "=", 2);
-
-                        if (environment_entry[0] == NULL || environment_entry[1] == NULL) {
-                                continue;
-                        }
-
-                        g_subprocess_launcher_setenv (launcher, environment_entry[0], environment_entry[1], FALSE);
-                }
-
-                /* Don't allow session specific environment variables from earlier sessions to
-                 * leak through */
-                for (i = 0; session_variables[i] != NULL; i++) {
-                        if (g_getenv (session_variables[i]) == NULL) {
-                                g_subprocess_launcher_unsetenv (launcher, session_variables[i]);
-                        }
-                }
-        }
-
         g_subprocess_launcher_setenv (launcher, "DISPLAY", state->display_name, TRUE);
         g_subprocess_launcher_setenv (launcher, "XAUTHORITY", state->auth_file, TRUE);
 
-        if (state->bus_address != NULL) {
-                g_subprocess_launcher_setenv (launcher, "DBUS_SESSION_BUS_ADDRESS", state->bus_address, TRUE);
-        }
-
         vt = g_getenv ("XDG_VTNR");
 
         if (vt != NULL) {
-                g_subprocess_launcher_setenv (launcher, "WINDOWPATH", vt, TRUE);
+                g_subprocess_launcher_setenv (launcher, "WINDOWPATH", g_getenv ("XDG_VTNR"), TRUE);
         }
 
         if (run_script) {
@@ -741,15 +588,30 @@ static gboolean
 register_display (State        *state,
                   GCancellable *cancellable)
 {
+        GdmDBusManager  *manager;
         GError          *error = NULL;
         gboolean         registered = FALSE;
         GVariantBuilder  details;
 
+        manager = gdm_dbus_manager_proxy_new_for_bus_sync (G_BUS_TYPE_SYSTEM,
+                                                           G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES |
+                                                           G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
+                                                           "org.gnome.DisplayManager",
+                                                           "/org/gnome/DisplayManager/Manager",
+                                                           cancellable,
+                                                           &error);
+
+        if (!manager) {
+                g_debug ("could not contact display manager: %s", error->message);
+                g_error_free (error);
+                goto out;
+        }
+
         g_variant_builder_init (&details, G_VARIANT_TYPE ("a{ss}"));
         g_variant_builder_add (&details, "{ss}", "session-type", "x11");
         g_variant_builder_add (&details, "{ss}", "x11-display-name", state->display_name);
 
-        registered = gdm_dbus_manager_call_register_display_sync (state->display_manager_proxy,
+        registered = gdm_dbus_manager_call_register_display_sync (manager,
                                                                   g_variant_builder_end (&details),
                                                                   cancellable,
                                                                   &error);
@@ -758,6 +620,8 @@ register_display (State        *state,
                 g_error_free (error);
         }
 
+out:
+        g_clear_object (&manager);
         return registered;
 }
 
@@ -775,14 +639,11 @@ clear_state (State **out_state)
         State *state = *out_state;
 
         g_clear_object (&state->cancellable);
-        g_clear_object (&state->bus_connection);
         g_clear_object (&state->session_subprocess);
         g_clear_object (&state->x_subprocess);
-        g_clear_pointer (&state->environment, g_strfreev);
         g_clear_pointer (&state->auth_file, g_free);
         g_clear_pointer (&state->display_name, g_free);
         g_clear_pointer (&state->main_loop, g_main_loop_unref);
-        g_clear_handle_id (&state->register_session_id, g_source_remove);
         *out_state = NULL;
 }
 
@@ -798,67 +659,21 @@ on_sigterm (State *state)
         return G_SOURCE_CONTINUE;
 }
 
-static gboolean
-register_session_timeout_cb (gpointer user_data)
-{
-        State *state;
-        GError *error = NULL;
-
-        state = (State *) user_data;
-
-        gdm_dbus_manager_call_register_session_sync (state->display_manager_proxy,
-                                                     g_variant_new ("a{sv}", NULL),
-                                                     state->cancellable,
-                                                     &error);
-
-        if (error != NULL) {
-                g_warning ("Could not register session: %s", error->message);
-                g_error_free (error);
-        }
-
-        return G_SOURCE_REMOVE;
-}
-
-static gboolean
-connect_to_display_manager (State *state)
-{
-        g_autoptr (GError) error = NULL;
-
-        state->display_manager_proxy = gdm_dbus_manager_proxy_new_for_bus_sync (
-                G_BUS_TYPE_SYSTEM,
-                G_DBUS_PROXY_FLAGS_DO_NOT_LOAD_PROPERTIES | G_DBUS_PROXY_FLAGS_DO_NOT_CONNECT_SIGNALS,
-                "org.gnome.DisplayManager",
-                "/org/gnome/DisplayManager/Manager",
-                state->cancellable,
-                &error);
-
-        if (state->display_manager_proxy == NULL) {
-                g_printerr ("gdm-x-session: could not contact display manager: %s\n",
-                            error->message);
-                return FALSE;
-        }
-
-        return TRUE;
-}
-
 int
 main (int    argc,
       char **argv)
 {
-        State           *state = NULL;
-        GOptionContext  *context = NULL;
+        State           *state;
+        GOptionContext  *context;
         static char    **args = NULL;
         static gboolean  run_script = FALSE;
         static gboolean  allow_remote_connections = FALSE;
         gboolean         debug = FALSE;
         gboolean         ret;
         int              exit_status = EX_OK;
-        static gboolean  register_session = FALSE;
-
         static GOptionEntry entries []   = {
                 { "run-script", 'r', 0, G_OPTION_ARG_NONE, &run_script, N_("Run program through /etc/gdm/Xsession wrapper script"), NULL },
                 { "allow-remote-connections", 'a', 0, G_OPTION_ARG_NONE, &allow_remote_connections, N_("Listen on TCP socket"), NULL },
-                { "register-session", 0, 0, G_OPTION_ARG_NONE, &register_session, "Register session after a delay", NULL },
                 { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &args, "", "" },
                 { NULL }
         };
@@ -889,7 +704,7 @@ main (int    argc,
         ret = gdm_settings_direct_init (state->settings, DATADIR "/gdm/gdm.schemas", "/");
 
         if (!ret) {
-                g_printerr ("Unable to initialize settings\n");
+                g_printerr ("Unable to initialize settings");
                 exit_status = EX_DATAERR;
                 goto out;
         }
@@ -907,7 +722,7 @@ main (int    argc,
         ret = spawn_x_server (state, allow_remote_connections, state->cancellable);
 
         if (!ret) {
-                g_printerr ("Unable to run X server\n");
+                g_printerr ("Unable to run X server");
                 exit_status = EX_SOFTWARE;
                 goto out;
         }
@@ -915,28 +730,15 @@ main (int    argc,
         ret = spawn_bus (state, state->cancellable);
 
         if (!ret) {
-                g_printerr ("Unable to run session message bus\n");
+                g_printerr ("Unable to run session message bus");
                 exit_status = EX_SOFTWARE;
                 goto out;
         }
 
-        import_environment (state, state->cancellable);
-
-        ret = update_bus_environment (state, state->cancellable);
-
-        if (!ret) {
-                g_printerr ("Unable to update bus environment\n");
-                exit_status = EX_SOFTWARE;
-                goto out;
-        }
-
-        if (!connect_to_display_manager (state))
-                goto out;
-
         ret = register_display (state, state->cancellable);
 
         if (!ret) {
-                g_printerr ("Unable to register display with display manager\n");
+                g_printerr ("Unable to register display with display manager");
                 exit_status = EX_SOFTWARE;
                 goto out;
         }
@@ -944,20 +746,11 @@ main (int    argc,
         ret = spawn_session (state, run_script, state->cancellable);
 
         if (!ret) {
-                g_printerr ("Unable to run session\n");
+                g_printerr ("Unable to run session");
                 exit_status = EX_SOFTWARE;
                 goto out;
         }
 
-        if (register_session) {
-                g_debug ("gdm-x-session: Will register session in %d seconds", REGISTER_SESSION_TIMEOUT);
-                state->register_session_id = g_timeout_add_seconds (REGISTER_SESSION_TIMEOUT,
-                                                                    register_session_timeout_cb,
-                                                                    state);
-        } else {
-                g_debug ("gdm-x-session: Session will register itself");
-        }
-
         g_main_loop_run (state->main_loop);
 
         /* Only use exit status of session if we're here because it exit */
@@ -967,11 +760,9 @@ main (int    argc,
         }
 
 out:
-        if (state != NULL) {
-                signal_subprocesses (state);
-                wait_on_subprocesses (state);
-                clear_state (&state);
-        }
+        signal_subprocesses (state);
+        wait_on_subprocesses (state);
+        clear_state (&state);
 
         return exit_status;
 }
